/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class java_awt_BDGraphics */

#ifndef _WIN32
#undef JNIEXPORT
#define JNIEXPORT static
#endif

#ifndef _Included_java_awt_BDGraphics
#define _Included_java_awt_BDGraphics
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     org_videolan_Logger
 * Method:    logN
 * Signature: (ZLjava/lang/String;ILjava/lang/String;)V

 */
JNIEXPORT void JNICALL Java_org_videolan_Logger_logN
  (JNIEnv *, jclass, jboolean, jstring, jint, jstring);

#ifdef __cplusplus
}
#endif
#endif
