/*
 * This file is part of libbluray
 * Copyright (C) 2010  William Hahne
 * Copyright (C) 2016-2019 Petri Hintukainen <phintuka@users.sourceforge.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 */

package org.videolan.media.content.playlist;

import org.bluray.media.PrimaryAudioControl;
import org.bluray.system.RegisterAccess;

import org.videolan.Libbluray;
import org.videolan.StreamInfo;
import org.videolan.TIClip;

public class PrimaryAudioControlImpl extends StreamControl implements PrimaryAudioControl {
    protected PrimaryAudioControlImpl(Handler player) {
        this.player = player;
    }

    protected StreamInfo[] getStreams() {
        TIClip ci = player.getCurrentClipInfo();
        if (ci == null)
            return null;
        return ci.getAudioStreams();
    }

    protected String getDefaultLanguage() {
        return languageFromInteger(RegisterAccess.getInstance().getPSR(RegisterAccess.PSR_LANG_CODE_AUDIO));
    }

    public int getCurrentStreamNumber() {
        return RegisterAccess.getInstance().getPSR(RegisterAccess.PSR_AUDIO_STN);
    }

    protected void setStreamNumber(int num) {
        Libbluray.writePSR(RegisterAccess.PSR_AUDIO_STN, num);
    }

    private Handler player;
}
