/*
 * This file is part of libbluray
 * Copyright (C) 2012  libbluray
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 */

package sun.awt.image;

import java.awt.image.ColorModel;
import java.awt.image.ImageObserver;
import java.awt.image.ImageProducer;
import java.awt.image.BufferedImage;
import java.awt.Graphics;

public interface BufferedImagePeer {
    void          flush();
    ColorModel    getColorModel();
    Graphics      getGraphics();
    int           getHeight();
    int           getHeight(ImageObserver observer);
    Object        getProperty(String name);
    Object        getProperty(String name, ImageObserver observer);
    String[]      getPropertyNames();
    int           getRGB(int x, int y);
    int[]         getRGB(int startX, int startY, int w, int h, int[] rgbArray, int offset, int scansize);
    ImageProducer getSource();
    BufferedImage getSubimage(int x, int y, int w, int h);
    int           getType();
    int           getWidth();
    int           getWidth(ImageObserver observer);
    void          setRGB(int x, int y, int rgb);
    void          setRGB(int startX, int startY, int w, int h, int[] rgbArray, int offset, int scansize);
}
